==> agenticraft-0.1.1.tar.gz: agenticraft-0.1.1/agenticraft/tools/web.py <==
"""Web search and scraping tools for agents.

Provides web search and basic web scraping capabilities.
Note: Real implementation requires API keys for search providers.
"""

import asyncio
import json
import re
from datetime import datetime
from typing import Any, Dict, List, Optional
from urllib.parse import quote_plus, urlparse

from ..core.tool import tool


@tool(
    name="web_search",
    description="Search the web for information. Returns relevant results with titles, snippets, and URLs."
)
async def web_search(
    query: str,
    num_results: int = 5,
    search_type: str = "general"
) -> List[Dict[str, Any]]:
    """Search the web for information.
    
    Args:
        query: Search query
        num_results: Number of results to return (max 10)
        search_type: Type of search (general, news, academic)
        
    Returns:
        List of search results
        
    Example:
        >>> await web_search("python programming", num_results=3)
        [
            {
                'title': 'Python Programming Language',
                'snippet': 'Official Python website...',
                'url': 'https://python.org',
                'date': '2024-01-15'
            },
            ...
        ]
    """
    # Limit results
    num_results = min(num_results, 10)
    
    # Note: This is a mock implementation
    # Real implementation would use APIs like:
    # - Google Custom Search API
    # - Bing Search API
    # - DuckDuckGo API
    # - SerpAPI
    
    # For demo purposes, return mock results
    mock_results = [
        {
            'title': f'Result {i+1} for: {query}',
            'snippet': f'This is a relevant snippet about {query}. '
                      f'It contains useful information that matches the search query...',
            'url': f'https://example.com/result{i+1}',
            'date': datetime.now().isoformat(),
            'relevance_score': 0.95 - (i * 0.05)
        }
        for i in range(num_results)
    ]
    
    # Add search type specific results
    if search_type == "news":
        for result in mock_results:
            result['source'] = 'Example News'
            result['category'] = 'Technology'
    elif search_type == "academic":
        for result in mock_results:
            result['authors'] = ['J. Doe', 'A. Smith']
            result['citations'] = 42
    
    return mock_results


@tool(
    name="extract_text",
    description="Extract clean text content from a webpage URL."
)
async def extract_text(
    url: str,
    include_links: bool = False,
    max_length: Optional[int] = None
) -> Dict[str, Any]:
    """Extract text content from a webpage.
    
    Args:
        url: URL to extract text from
        include_links: Whether to include hyperlinks
        max_length: Maximum text length to return
        
    Returns:
        Dictionary with extracted content
        
    Example:
        >>> await extract_text("https://example.com/article")
        {
            'url': 'https://example.com/article',
            'title': 'Article Title',
            'text': 'Article content...',
            'links': [],
            'word_count': 500
        }
    """
    # Validate URL
    parsed = urlparse(url)
    if not parsed.scheme or not parsed.netloc:
        raise ValueError(f"Invalid URL: {url}")
    
    # Check for supported protocols
    if parsed.scheme not in ['http', 'https']:
        raise ValueError(f"Invalid URL: {url}")  # Only HTTP/HTTPS supported
    
    # Note: Real implementation would use:
    # - aiohttp for async HTTP requests
    # - BeautifulSoup or lxml for HTML parsing
    # - Readability or newspaper3k for article extraction
    
    # Mock implementation
    mock_content = {
        'url': url,
        'title': f'Page Title from {parsed.netloc}',
        'text': f'This is the extracted text content from {url}. '
               f'It contains the main article or page content without '
               f'navigation, ads, or other clutter. The extraction process '
               f'identifies the primary content area and extracts clean text.',
        'word_count': 42,
        'language': 'en',
        'published_date': None
    }
    
    if include_links:
        mock_content['links'] = [
            {'text': 'Related Article', 'url': f'{url}/related'},
            {'text': 'More Information', 'url': f'{url}/more'}
        ]
    
    # Truncate if needed
    if max_length and len(mock_content['text']) > max_length:
        mock_content['text'] = mock_content['text'][:max_length] + '...'
        mock_content['truncated'] = True
    
    return mock_content


@tool(
    name="get_page_metadata",
    description="Extract metadata from a webpage including title, description, and Open Graph data."
)
async def get_page_metadata(url: str) -> Dict[str, Any]:
    """Get metadata from a webpage.
    
    Args:
        url: URL to analyze
        
    Returns:
        Dictionary with page metadata
        
    Example:
        >>> await get_page_metadata("https://example.com")
        {
            'url': 'https://example.com',
            'title': 'Example Domain',
            'description': 'Example Domain for Documentation',
            'image': 'https://example.com/image.jpg',
            'type': 'website'
        }
    """
    # Validate URL
    parsed = urlparse(url)
    if not parsed.scheme or not parsed.netloc:
        raise ValueError(f"Invalid URL: {url}")
    
    # Check for supported protocols
    if parsed.scheme not in ['http', 'https']:
        raise ValueError(f"Invalid URL: {url}")  # Only HTTP/HTTPS supported
    
    # Mock implementation
    # Real implementation would parse meta tags, Open Graph, Twitter Cards, etc.
    return {
        'url': url,
        'title': f'{parsed.netloc} - Home',
        'description': f'Welcome to {parsed.netloc}',
        'keywords': ['example', 'demo', 'website'],
        'author': None,
        'image': f'{parsed.scheme}://{parsed.netloc}/og-image.jpg',
        'type': 'website',
        'locale': 'en_US',
        'site_name': parsed.netloc.replace('.', ' ').title()
    }


@tool(
    name="check_url",
    description="Check if a URL is accessible and get basic information."
)
async def check_url(url: str) -> Dict[str, Any]:
    """Check URL accessibility and get basic info.
    
    Args:
        url: URL to check
        
    Returns:
        Dictionary with URL status and info
        
    Example:
        >>> await check_url("https://example.com")
        {
            'url': 'https://example.com',
            'accessible': True,
            'status_code': 200,
            'content_type': 'text/html',
            'response_time_ms': 150
        }
    """
    # Validate URL
    parsed = urlparse(url)
    if not parsed.scheme or not parsed.netloc:
        raise ValueError(f"Invalid URL: {url}")
    
    # Check for supported protocols
    if parsed.scheme not in ['http', 'https']:
        raise ValueError(f"Invalid URL: {url}")  # Only HTTP/HTTPS supported
    
    # Mock implementation
    # Real implementation would make HEAD request
    return {
        'url': url,
        'accessible': True,
        'status_code': 200,
        'content_type': 'text/html; charset=utf-8',
        'content_length': 1024 * 10,  # 10KB
        'response_time_ms': 150,
        'ssl_valid': parsed.scheme == 'https',
        'redirects': []
    }


# Note for real implementation:
"""
To implement real web search and scraping:

1. Web Search:
   - Use search APIs (Google, Bing, DuckDuckGo)
   - Implement rate limiting and caching
   - Handle API authentication

2. Web Scraping:
   - Use aiohttp for async HTTP requests
   - Use BeautifulSoup4 or lxml for parsing
   - Implement robots.txt compliance
   - Add user-agent headers
   - Handle JavaScript-rendered content (Playwright/Selenium)

3. Security:
   - Validate and sanitize URLs
   - Implement request timeouts
   - Limit response sizes
   - Block internal/private IP ranges

4. Performance:
   - Connection pooling
   - Response caching
   - Concurrent request limits
"""

==> agenticraft-0.2.0a0.tar.gz: agenticraft-0.2.0a0/agenticraft/tools/web.py <==
"""Web search and scraping tools for agents.

Provides web search and basic web scraping capabilities.
Note: Real implementation requires API keys for search providers.
"""

from datetime import datetime
from typing import Any
from urllib.parse import urlparse

from ..core.tool import tool


@tool(
    name="web_search",
    description="Search the web for information. Returns relevant results with titles, snippets, and URLs.",
)
async def web_search(
    query: str, num_results: int = 5, search_type: str = "general"
) -> list[dict[str, Any]]:
    """Search the web for information.

    Args:
        query: Search query
        num_results: Number of results to return (max 10)
        search_type: Type of search (general, news, academic)

    Returns:
        List of search results

    Example:
        >>> await web_search("python programming", num_results=3)
        [
            {
                'title': 'Python Programming Language',
                'snippet': 'Official Python website...',
                'url': 'https://python.org',
                'date': '2024-01-15'
            },
            ...
        ]
    """
    # Limit results
    num_results = min(num_results, 10)

    # Note: This is a mock implementation
    # Real implementation would use APIs like:
    # - Google Custom Search API
    # - Bing Search API
    # - DuckDuckGo API
    # - SerpAPI

    # For demo purposes, return mock results
    mock_results = [
        {
            "title": f"Result {i+1} for: {query}",
            "snippet": f"This is a relevant snippet about {query}. "
            f"It contains useful information that matches the search query...",
            "url": f"https://example.com/result{i+1}",
            "date": datetime.now().isoformat(),
            "relevance_score": 0.95 - (i * 0.05),
        }
        for i in range(num_results)
    ]

    # Add search type specific results
    if search_type == "news":
        for result in mock_results:
            result["source"] = "Example News"
            result["category"] = "Technology"
    elif search_type == "academic":
        for result in mock_results:
            result["authors"] = ["J. Doe", "A. Smith"]
            result["citations"] = 42

    return mock_results


@tool(name="extract_text", description="Extract clean text content from a webpage URL.")
async def extract_text(
    url: str, include_links: bool = False, max_length: int | None = None
) -> dict[str, Any]:
    """Extract text content from a webpage.

    Args:
        url: URL to extract text from
        include_links: Whether to include hyperlinks
        max_length: Maximum text length to return

    Returns:
        Dictionary with extracted content

    Example:
        >>> await extract_text("https://example.com/article")
        {
            'url': 'https://example.com/article',
            'title': 'Article Title',
            'text': 'Article content...',
            'links': [],
            'word_count': 500
        }
    """
    # Validate URL
    parsed = urlparse(url)
    if not parsed.scheme or not parsed.netloc:
        raise ValueError(f"Invalid URL: {url}")

    # Check for supported protocols
    if parsed.scheme not in ["http", "https"]:
        raise ValueError(f"Invalid URL: {url}")  # Only HTTP/HTTPS supported

    # Note: Real implementation would use:
    # - aiohttp for async HTTP requests
    # - BeautifulSoup or lxml for HTML parsing
    # - Readability or newspaper3k for article extraction

    # Mock implementation
    mock_content = {
        "url": url,
        "title": f"Page Title from {parsed.netloc}",
        "text": f"This is the extracted text content from {url}. "
        f"It contains the main article or page content without "
        f"navigation, ads, or other clutter. The extraction process "
        f"identifies the primary content area and extracts clean text.",
        "word_count": 42,
        "language": "en",
        "published_date": None,
    }

    if include_links:
        mock_content["links"] = [
            {"text": "Related Article", "url": f"{url}/related"},
            {"text": "More Information", "url": f"{url}/more"},
        ]

    # Truncate if needed
    if max_length and len(mock_content["text"]) > max_length:
        mock_content["text"] = mock_content["text"][:max_length] + "..."
        mock_content["truncated"] = True

    return mock_content


@tool(
    name="get_page_metadata",
    description="Extract metadata from a webpage including title, description, and Open Graph data.",
)
async def get_page_metadata(url: str) -> dict[str, Any]:
    """Get metadata from a webpage.

    Args:
        url: URL to analyze

    Returns:
        Dictionary with page metadata

    Example:
        >>> await get_page_metadata("https://example.com")
        {
            'url': 'https://example.com',
            'title': 'Example Domain',
            'description': 'Example Domain for Documentation',
            'image': 'https://example.com/image.jpg',
            'type': 'website'
        }
    """
    # Validate URL
    parsed = urlparse(url)
    if not parsed.scheme or not parsed.netloc:
        raise ValueError(f"Invalid URL: {url}")

    # Check for supported protocols
    if parsed.scheme not in ["http", "https"]:
        raise ValueError(f"Invalid URL: {url}")  # Only HTTP/HTTPS supported

    # Mock implementation
    # Real implementation would parse meta tags, Open Graph, Twitter Cards, etc.
    return {
        "url": url,
        "title": f"{parsed.netloc} - Home",
        "description": f"Welcome to {parsed.netloc}",
        "keywords": ["example", "demo", "website"],
        "author": None,
        "image": f"{parsed.scheme}://{parsed.netloc}/og-image.jpg",
        "type": "website",
        "locale": "en_US",
        "site_name": parsed.netloc.replace(".", " ").title(),
    }


@tool(
    name="check_url",
    description="Check if a URL is accessible and get basic information.",
)
async def check_url(url: str) -> dict[str, Any]:
    """Check URL accessibility and get basic info.

    Args:
        url: URL to check

    Returns:
        Dictionary with URL status and info

    Example:
        >>> await check_url("https://example.com")
        {
            'url': 'https://example.com',
            'accessible': True,
            'status_code': 200,
            'content_type': 'text/html',
            'response_time_ms': 150
        }
    """
    # Validate URL
    parsed = urlparse(url)
    if not parsed.scheme or not parsed.netloc:
        raise ValueError(f"Invalid URL: {url}")

    # Check for supported protocols
    if parsed.scheme not in ["http", "https"]:
        raise ValueError(f"Invalid URL: {url}")  # Only HTTP/HTTPS supported

    # Mock implementation
    # Real implementation would make HEAD request
    return {
        "url": url,
        "accessible": True,
        "status_code": 200,
        "content_type": "text/html; charset=utf-8",
        "content_length": 1024 * 10,  # 10KB
        "response_time_ms": 150,
        "ssl_valid": parsed.scheme == "https",
        "redirects": [],
    }


# Note for real implementation:
"""
To implement real web search and scraping:

1. Web Search:
   - Use search APIs (Google, Bing, DuckDuckGo)
   - Implement rate limiting and caching
   - Handle API authentication

2. Web Scraping:
   - Use aiohttp for async HTTP requests
   - Use BeautifulSoup4 or lxml for parsing
   - Implement robots.txt compliance
   - Add user-agent headers
   - Handle JavaScript-rendered content (Playwright/Selenium)

3. Security:
   - Validate and sanitize URLs
   - Implement request timeouts
   - Limit response sizes
   - Block internal/private IP ranges

4. Performance:
   - Connection pooling
   - Response caching
   - Concurrent request limits
"""
